export declare const InternalTypeSymbol: unique symbol;
export declare type JsonValue = {
    [property: string]: JsonValue;
} | boolean | readonly JsonValue[] | null | number | string;
declare type PartialRequire<T, U extends keyof T> = T & Required<Pick<T, U>>;
declare type UnionToIntersection<U> = (U extends any ? (k: U) => void : never) extends (k: infer I) => void ? I : never;
export interface SchemaLike {
    [InternalTypeSymbol]: unknown;
}
declare type SimpleType = "array" | "boolean" | "integer" | "null" | "number" | "object" | "string";
declare type ConstConstraint<Const extends JsonValue | undefined> = Const extends JsonValue ? Const : unknown;
declare type SimpleTypeConstraint<Type extends SimpleType | undefined> = Type extends "array" ? Array<JsonValue> : Type extends "boolean" ? boolean : Type extends "integer" | "number" ? number : Type extends "null" ? null : Type extends "object" ? {
    [k: string]: unknown;
} : Type extends "string" ? string : unknown;
declare type EnumConstraint<Enum extends readonly JsonValue[] | undefined> = Enum extends readonly JsonValue[] ? Enum[number] : unknown;
declare type PropertiesConstraint<Properties extends {
    [k: string]: SchemaLike;
} | undefined, Required extends readonly (keyof Properties)[] | undefined> = Properties extends {
    [k: string]: SchemaLike;
} ? Required extends readonly (keyof Properties)[] ? PartialRequire<Partial<{
    [P in keyof Properties]: Properties[P][typeof InternalTypeSymbol];
}>, Required[number]> : Partial<{
    [P in keyof Properties]: Properties[P][typeof InternalTypeSymbol];
}> : unknown;
declare type AdditionalPropertiesConstraint<AdditionalProperties extends SchemaLike | undefined> = AdditionalProperties extends SchemaLike ? {
    [k: string]: AdditionalProperties[typeof InternalTypeSymbol];
} : unknown;
declare type ItemsConstraint<Items extends (SchemaLike | readonly SchemaLike[]) | undefined, AdditionalItems extends SchemaLike | undefined> = Items extends SchemaLike ? Array<Items[typeof InternalTypeSymbol]> : Items extends readonly SchemaLike[] ? AdditionalItems extends SchemaLike ? Array<Items[number][typeof InternalTypeSymbol] | AdditionalItems[typeof InternalTypeSymbol]> : Array<Items[number][typeof InternalTypeSymbol]> : unknown;
declare type AllOfConstraint<AllOf extends readonly SchemaLike[] | undefined> = AllOf extends readonly SchemaLike[] ? UnionToIntersection<AllOf[number][typeof InternalTypeSymbol]> : unknown;
declare type AnyOfConstraint<AnyOf extends readonly SchemaLike[] | undefined> = AnyOf extends readonly SchemaLike[] ? AnyOf[number][typeof InternalTypeSymbol] : unknown;
declare type OneOfConstraint<OneOf extends readonly SchemaLike[] | undefined> = AnyOfConstraint<OneOf>;
declare type IfThenElseConstraint<Then extends SchemaLike | undefined, Else extends SchemaLike | undefined> = Then extends SchemaLike ? Else extends SchemaLike ? Then[typeof InternalTypeSymbol] | Else[typeof InternalTypeSymbol] : unknown : unknown;
interface Schema<Type extends SimpleType | undefined = undefined, // type can be either a single type or a list of types (TODO allow it to be a list of types) (| readonly SimpleType[])
Properties extends {
    [k: string]: SchemaLike;
} | undefined = undefined, Items extends (SchemaLike | readonly SchemaLike[]) | undefined = undefined, AdditionalItems extends SchemaLike | undefined = undefined, AdditionalProperties extends SchemaLike | undefined = undefined, Required extends readonly (keyof Properties)[] | undefined = undefined, Const extends JsonValue | undefined = undefined, Enum extends readonly JsonValue[] | undefined = undefined, AllOf extends readonly SchemaLike[] | undefined = undefined, AnyOf extends readonly SchemaLike[] | undefined = undefined, OneOf extends readonly SchemaLike[] | undefined = undefined, Not extends SchemaLike | undefined = undefined, // not yet enforced
If extends SchemaLike | undefined = undefined, Then extends SchemaLike | undefined = undefined, Else extends SchemaLike | undefined = undefined, Definitions extends {
    [k: string]: SchemaLike;
} | undefined = undefined, // not yet enforced via refs
Ref extends string | undefined = undefined, // ref just gives unknown types for now. TODO: connect with definitions and get proper recursive types here.
Dependencies extends {
    [k in keyof Properties]: SchemaLike | keyof Properties[];
} | undefined = undefined, CalculatedType = ConstConstraint<Const> & SimpleTypeConstraint<Type> & EnumConstraint<Enum> & PropertiesConstraint<Properties, Required> & AdditionalPropertiesConstraint<AdditionalProperties> & ItemsConstraint<Items, AdditionalItems> & AllOfConstraint<AllOf> & AnyOfConstraint<AnyOf> & OneOfConstraint<OneOf> & IfThenElseConstraint<Then, Else>> {
    $id?: string;
    $schema?: "http://json-schema.org/draft-07/schema#";
    $ref?: Ref;
    $comment?: string;
    title?: string;
    description?: string;
    default?: CalculatedType;
    readOnly?: boolean;
    examples?: JsonValue[];
    multipleOf?: Type extends "number" | "integer" ? number : never;
    maximum?: Type extends "number" | "integer" ? number : never;
    exclusiveMaximum?: Type extends "number" | "integer" ? number : never;
    minimum?: Type extends "number" | "integer" ? number : never;
    exclusiveMinimum?: Type extends "number" | "integer" ? number : never;
    minLength?: Type extends "string" ? number : never;
    maxLength?: Type extends "string" ? number : never;
    pattern?: Type extends "string" ? string : never;
    additionalItems?: Type extends "array" ? Items extends SchemaLike ? never : AdditionalItems : never;
    items?: Type extends "array" ? Items : never;
    maxItems?: Type extends "array" ? number : never;
    minItems?: Type extends "array" ? number : never;
    uniqueItems?: Type extends "array" ? boolean : never;
    contains?: Type extends "array" ? SchemaLike : never;
    maxProperties?: Type extends "object" ? number : never;
    minProperties?: Type extends "object" ? number : never;
    required?: Type extends "object" ? Required : never;
    additionalProperties?: Type extends "object" ? AdditionalProperties : never;
    definitions?: Definitions;
    properties?: Type extends "object" ? Properties : never;
    patternProperties?: Type extends "object" ? {
        [k: string]: SchemaLike;
    } : never;
    dependencies?: Type extends "object" ? Dependencies : never;
    propertyNames?: Type extends "object" ? SchemaLike : never;
    const?: Const;
    enum?: Enum;
    type?: Type;
    format?: Type extends "string" ? string : never;
    contentMediaType?: Type extends "string" ? string : never;
    contentEncoding?: Type extends "string" ? string : never;
    if?: Then extends SchemaLike ? SchemaLike : Else extends SchemaLike ? SchemaLike : never;
    then?: If extends SchemaLike ? SchemaLike : never;
    else?: If extends SchemaLike ? SchemaLike : never;
    allOf?: AllOf;
    anyOf?: AnyOf;
    oneOf?: OneOf;
    not?: Not;
    [InternalTypeSymbol]?: CalculatedType;
}
export declare const createSchema: <Type extends "string" | "number" | "boolean" | "object" | "array" | "integer" | "null" | undefined = undefined, Properties extends {
    [k: string]: SchemaLike;
} | undefined = undefined, Items extends SchemaLike | readonly SchemaLike[] | undefined = undefined, AdditionalItems extends SchemaLike | undefined = undefined, AdditionalProperties extends SchemaLike | undefined = undefined, Required_1 extends readonly (keyof Properties)[] | undefined = undefined, Const extends string | number | boolean | {
    [property: string]: JsonValue;
} | readonly JsonValue[] | null | undefined = undefined, Enum extends readonly JsonValue[] | undefined = undefined, AllOf extends readonly SchemaLike[] | undefined = undefined, AnyOf extends readonly SchemaLike[] | undefined = undefined, OneOf extends readonly SchemaLike[] | undefined = undefined, Not extends SchemaLike | undefined = undefined, If extends SchemaLike | undefined = undefined, Then extends SchemaLike | undefined = undefined, Else extends SchemaLike | undefined = undefined, Definitions extends {
    [k: string]: SchemaLike;
} | undefined = undefined, Ref extends string | undefined = undefined, Dependencies extends { [k in keyof Properties]: number | SchemaLike | "length" | "toString" | "toLocaleString" | "concat" | "join" | "slice" | "indexOf" | "lastIndexOf" | "every" | "some" | "forEach" | "map" | "filter" | "reduce" | "reduceRight" | "find" | "findIndex" | "entries" | "keys" | "values" | "pop" | "push" | "reverse" | "shift" | "sort" | "splice" | "unshift" | "fill" | "copyWithin"; } | undefined = undefined>(schema: Schema<Type, Properties, Items, AdditionalItems, AdditionalProperties, Required_1, Const, Enum, AllOf, AnyOf, OneOf, Not, If, Then, Else, Definitions, Ref, Dependencies, ConstConstraint<Const> & SimpleTypeConstraint<Type> & EnumConstraint<Enum> & PropertiesConstraint<Properties, Required_1> & AdditionalPropertiesConstraint<AdditionalProperties> & ItemsConstraint<Items, AdditionalItems> & AllOfConstraint<AllOf> & AnyOfConstraint<AnyOf> & AnyOfConstraint<OneOf> & IfThenElseConstraint<Then, Else>>) => {
    [InternalTypeSymbol]: NonNullable<ConstConstraint<Const> & SimpleTypeConstraint<Type> & EnumConstraint<Enum> & PropertiesConstraint<Properties, Required_1> & AdditionalPropertiesConstraint<AdditionalProperties> & ItemsConstraint<Items, AdditionalItems> & AllOfConstraint<AllOf> & AnyOfConstraint<AnyOf> & AnyOfConstraint<OneOf> & IfThenElseConstraint<Then, Else>>;
    $id?: string | undefined;
    $schema?: "http://json-schema.org/draft-07/schema#" | undefined;
    $ref?: Ref | undefined;
    $comment?: string | undefined;
    title?: string | undefined;
    description?: string | undefined;
    default?: (ConstConstraint<Const> & SimpleTypeConstraint<Type> & EnumConstraint<Enum> & PropertiesConstraint<Properties, Required_1> & AdditionalPropertiesConstraint<AdditionalProperties> & ItemsConstraint<Items, AdditionalItems> & AllOfConstraint<AllOf> & AnyOfConstraint<AnyOf> & AnyOfConstraint<OneOf> & IfThenElseConstraint<Then, Else>) | undefined;
    readOnly?: boolean | undefined;
    examples?: JsonValue[] | undefined;
    multipleOf?: (Type extends "number" | "integer" ? number : never) | undefined;
    maximum?: (Type extends "number" | "integer" ? number : never) | undefined;
    exclusiveMaximum?: (Type extends "number" | "integer" ? number : never) | undefined;
    minimum?: (Type extends "number" | "integer" ? number : never) | undefined;
    exclusiveMinimum?: (Type extends "number" | "integer" ? number : never) | undefined;
    minLength?: (Type extends "string" ? number : never) | undefined;
    maxLength?: (Type extends "string" ? number : never) | undefined;
    pattern?: (Type extends "string" ? string : never) | undefined;
    additionalItems?: (Type extends "array" ? Items extends SchemaLike ? never : AdditionalItems : never) | undefined;
    items?: (Type extends "array" ? Items : never) | undefined;
    maxItems?: (Type extends "array" ? number : never) | undefined;
    minItems?: (Type extends "array" ? number : never) | undefined;
    uniqueItems?: (Type extends "array" ? boolean : never) | undefined;
    contains?: (Type extends "array" ? SchemaLike : never) | undefined;
    maxProperties?: (Type extends "object" ? number : never) | undefined;
    minProperties?: (Type extends "object" ? number : never) | undefined;
    required?: (Type extends "object" ? Required_1 : never) | undefined;
    additionalProperties?: (Type extends "object" ? AdditionalProperties : never) | undefined;
    definitions?: Definitions | undefined;
    properties?: (Type extends "object" ? Properties : never) | undefined;
    patternProperties?: (Type extends "object" ? {
        [k: string]: SchemaLike;
    } : never) | undefined;
    dependencies?: (Type extends "object" ? Dependencies : never) | undefined;
    propertyNames?: (Type extends "object" ? SchemaLike : never) | undefined;
    const?: Const | undefined;
    enum?: Enum | undefined;
    type?: Type | undefined;
    format?: (Type extends "string" ? string : never) | undefined;
    contentMediaType?: (Type extends "string" ? string : never) | undefined;
    contentEncoding?: (Type extends "string" ? string : never) | undefined;
    if?: (Then extends SchemaLike ? SchemaLike : Else extends SchemaLike ? SchemaLike : never) | undefined;
    then?: (If extends SchemaLike ? SchemaLike : never) | undefined;
    else?: (If extends SchemaLike ? SchemaLike : never) | undefined;
    allOf?: AllOf | undefined;
    anyOf?: AnyOf | undefined;
    oneOf?: OneOf | undefined;
    not?: Not | undefined;
};
export {};
